@@ -85,8 +85,7 @@ static void haio_invoke_cb(void *arg) {
   PROTECT(call = Rf_lcons(nano_ResolveSymbol, Rf_cons(status, R_NilValue)));
   Rf_eval(call, NANO_ENCLOS(x));
   UNPROTECT(1);
-  // unreliable to release linked list node from later cb, just free the payload
-  SET_TAG(node, R_NilValue);
+  nano_ReleaseObject(node);
 
 }
 
@@ -124,9 +123,6 @@ static void haio_finalizer(SEXP xptr) {
   nng_http_client_free(handle->cli);
   nng_url_free(handle->url);
   R_Free(handle);
-  // release linked list node if cb has already run
-  if (xp->cb != NULL && TAG((SEXP) xp->cb) == R_NilValue)
-    nano_ReleaseObject((SEXP) xp->cb);
   R_Free(xp);
 
 }
