@@ -87,6 +87,7 @@ test_zero(n$send(1:5, mode = "r"))
 test_equal(length(n1$recv("int", block = 500)), 5L)
 test_true(is_aio(saio <- n1$send_aio(paste(replicate(5, random(1e3L)), collapse = ""), mode = 1L, timeout = 900)))
 test_print(saio)
+if (later) test_null(.keep(saio, new.env()))
 test_class("sendAio", call_aio(saio))
 test_zero(saio$result)
 test_error(n$send("wrong mode", mode = "none"), "mode")
@@ -206,20 +207,11 @@ test_error(req$opt("false", NULL), "supported")
 test_error(req$opt("false", TRUE), "supported")
 test_error(req$opt("false", list()), "type")
 
-test_class("recvAio", r <- recv_aio(rep, timeout = 500))
-test_zero(req$send("", block = 500))
-test_print(p <- tryCatch(collect_pipe(r), error = function(e) NULL))
-if (!is.null(p)) test_class("nano", p)
-test_true(.mark())
-test_class("sendAio", r <- send_aio(if (is_nano(p)) p else rep, "", timeout = 500))
-if (later) test_null(.keep(r, new.env()))
-test_error(collect_pipe(r), "valid")
-test_equal(req$recv(mode = 8L, block = 500)[4L], 1L)
-test_true(!.mark(FALSE))
-
 test_class("nanoContext", ctx <- context(rep))
 test_print(ctx)
+test_true(.mark())
 test_class("sendAio", csaio <- req$send_aio(data.frame(), mode = "seria", timeout = 500))
+test_true(!.mark(FALSE))
 test_zero(call_aio_(csaio)$result)
 test_class("recvAio", craio <- recv_aio(ctx, timeout = 500))
 test_type("list", collect_aio(craio))
@@ -255,6 +247,7 @@ test_type("complex", call_aio(rek)[["data"]])
 test_class("recvAio", rek <- request(req$context, c(1+3i, 4+2i), send_mode = "serial", recv_mode = "serial", timeout = 500))
 test_zero(reply(ctx, execute = identity, recv_mode = 1L, send_mode = 1L, timeout = 500))
 test_type("complex", call_aio(rek)[["data"]])
+test_type("integer", rek[["aio"]])
 
 test_type("list", cfg <- serial_config(class = "custom", sfunc = function(x) raw(1L), ufunc = as.integer, vec = FALSE))
 test_equal(length(cfg), 4L)
@@ -287,6 +280,7 @@ test_notnull(cs$data)
 test_type("externalptr", ctxn <- .context(rep))
 test_class("recvAio", cr <- recv_aio(ctxn, cv = cv, timeout = 500))
 test_equal(call_aio(cr)$data, "test")
+test_type("integer", cr$aio)
 test_type("integer", send(ctxn, TRUE, mode = 0L, block = FALSE))
 test_type("externalptr", ctxn <- .context(rep))
 test_class("recvAio", cs <- request(.context(req$socket), data = TRUE, cv = NA))
@@ -308,7 +302,6 @@ test_error(until_(err, 10), "valid")
 test_error(cv_value(err), "valid")
 test_error(cv_reset(err), "valid")
 test_error(cv_signal(err), "valid")
-test_error(collect_pipe(err), "valid")
 test_error(pipe_notify(err, cv), "valid Socket")
 test_error(pipe_notify(rep, err), "valid Condition Variable")
 test_error(pipe_notify(rep, cv, err), "valid Condition Variable")
@@ -321,8 +314,6 @@ test_type("externalptr", rep$dialer[[1L]])
 test_zero(close(ctx))
 test_equal(suppressWarnings(close(ctx)), 7L)
 test_zero(close(rep))
-if (is_nano(p)) test_equal(reap(p), 12L)
-if (is_nano(p)) test_equal(suppressWarnings(close(p)), 12L)
 
 test_class("nanoObject", pub <- nano("pub", listen = "inproc://ps"))
 test_class("nanoObject", sub <- nano("sub", dial = "inproc://ps", autostart = NA))
@@ -368,9 +359,6 @@ test_class("nano", surv$survey_time(5000))
 test_type("externalptr", surv$context_open())
 test_type("externalptr", resp$context_open())
 test_class("nano", surv$survey_time(value = 2000))
-test_class("recvAio", r <- surv$recv_aio())
-test_error(collect_pipe(r), "11 | Incorrect state")
-test_equal(r$data, 11L)
 test_zero(surv$context_close())
 test_zero(resp$context_close())
 test_zero(surv$close())
@@ -379,11 +367,41 @@ test_true(!wait(cv))
 test_true(!wait(cv2))
 test_class("errorValue", resp$recv())
 
+test_class("nanoObject", poly <- nano(protocol = "poly", listen = "inproc://polytest"))
+test_equal(formals(poly$send)$pipe, 0L)
+test_equal(formals(poly$send_aio)$pipe, 0L)
+test_zero(poly$close())
+
+test_zero(cv_reset(cv))
+test_class("nanoSocket", poly <- socket(protocol = "poly"))
+test_class("nanoSocket", poly1 <- socket(protocol = "poly"))
+test_class("nanoSocket", poly2 <- socket(protocol = "poly"))
+test_class("nanoMonitor", m <- monitor(poly, cv))
+test_print(m)
+test_zero(listen(poly))
+test_null(read_monitor(m))
+test_zero(dial(poly1))
+test_zero(dial(poly2))
+test_true(wait(cv))
+test_true(wait(cv))
+test_equal(length(pipes <- read_monitor(m)), 2L)
+test_zero(send_aio(poly, "one", timeout = 500, pipe = pipes[1L])[])
+test_zero(send(poly, "two", block = 500, pipe = pipes[2L]))
+test_type("character", recv(poly1, block = 500))
+test_type("character", recv(poly2, block = 500))
+test_zero(reap(poly2))
+test_zero(reap(poly1))
+test_true(wait(cv))
+test_type("integer", read_monitor(m))
+test_error(read_monitor(poly), "valid Monitor")
+test_error(monitor("socket", "cv"), "valid Socket")
+test_error(monitor(poly, "cv"), "valid Condition Variable")
+test_zero(reap(poly))
+
 test_class("nanoSocket", bus <- socket(protocol = "bus"))
 test_class("nanoSocket", push <- socket(protocol = "push"))
 test_class("nanoSocket", pull <- socket(protocol = "pull"))
 test_class("nanoSocket", pair <- socket(protocol = "pair"))
-test_class("nanoSocket", poly <- socket(protocol = "poly"))
 test_class("nano", bus)
 test_equal(suppressWarnings(listen(bus, url = "test")), 3L)
 test_equal(suppressWarnings(dial(bus, url = "test")), 3L)
@@ -394,7 +412,6 @@ test_equal(suppressWarnings(close(bus)), 7L)
 test_zero(close(push))
 test_zero(close(pull))
 test_zero(reap(pair))
-test_zero(reap(poly))
 test_error(socket(protocol = "newprotocol"), "protocol")
 test_error(socket(dial = "test"), "argument")
 test_error(socket(listen = "test"), "argument")
@@ -524,36 +541,6 @@ test_zero(close(s1))
 test_zero(close(s2))
 test_zero(close(s3))
 
-test_type("externalptr", cv <- cv())
-test_error(.dispatcher(host = "inproc://hostdisp", url = "inproc://disp/1", tls = ""), "not a valid TLS Configuration")
-test_type("externalptr", disp <- .dispatcher(host = "inproc://hostdisp", url = "inproc://disp/1", tls = NULL))
-test_zero(.online(disp))
-test_null(.online("a"))
-test_class("nanoSocket", s <- socket(protocol = "rep"))
-test_zero(pipe_notify(s, cv, add = TRUE))
-test_zero(dial(s, url = "inproc://disp/1"))
-test_true(wait(cv))
-test_zero(send(disp, NULL, block = 500L))
-test_null(recv(s, block = 500L))
-test_zero(send(s, TRUE, block = 500L))
-test_true(recv(disp, block = 500L))
-test_zero(send(disp, NULL, block = 500L))
-test_null(recv(s, block = 500L))
-test_zero(reap(s))
-Sys.sleep(0.1)
-test_class("nanoSocket", s <- socket(protocol = "rep"))
-test_zero(pipe_notify(s, cv, add = TRUE))
-test_zero(dial(s, url = "inproc://disp/1"))
-test_true(wait(cv))
-test_zero(send(disp, TRUE, block = 500L))
-test_true(recv(s, block = 500L))
-test_true(.mark())
-test_zero(send(s, NULL, block = 500L))
-test_true(!.mark(FALSE))
-test_null(recv(disp, block = 500L))
-test_zero(reap(s))
-rm(disp)
-
 test_equal(nanonext:::.DollarNames.ncurlAio(NULL, "sta"), "status")
 test_equal(nanonext:::.DollarNames.recvAio(NULL, "dat"), "data")
 test_equal(nanonext:::.DollarNames.sendAio(NULL, "r"), "result")
@@ -586,7 +573,6 @@ test_error(send(fakestream, "test"), "valid")
 test_error(send_aio(fakestream, "test"), "valid")
 test_error(recv(fakestream), "valid")
 test_error(recv_aio(fakestream), "valid")
-test_error(collect_pipe(fakestream), "valid")
 test_error(opt(fakestream, name = "test") <- "test", "valid")
 test_error(opt(fakestream, name = "test"), "valid")
 test_error(close(fakestream), "active Stream")
@@ -596,8 +582,6 @@ test_error(close(fakedial), "valid Dialer")
 fakelist <- `class<-`("test", "nanoListener")
 test_error(start(fakelist), "valid Listener")
 test_error(close(fakelist), "valid Listener")
-fakepipe <- `class<-`("test", "nanoPipe")
-test_error(close(fakepipe), "valid Pipe")
 unres <- `class<-`(NA, "unresolvedValue")
 test_true(!unresolved(unres))
 test_print(unres)
@@ -662,6 +646,9 @@ if (promises) later::run_now()
 if (promises) test_zero(close(s1))
 if (promises) test_zero(close(s))
 if (promises) later::run_now()
+
+test_true(.interrupt())
+test_true(!.interrupt(FALSE))
 test_true(!identical(get0(".Random.seed"), {.advance(); .Random.seed}))
 test_type("integer", .Call(nanonext:::rnng_traverse_precious))
 if (Sys.info()[["sysname"]] == "Linux") {
